
#pragma once

#include <Arduino.h>

//BME280モジュールのスレーブアドレス(7bit)
#define	BME280_SLAVE_ADDRESS	0x76

//SettingOption	お勧めセッティング → WeatherMonitoring固定で運用することとしたので廃止
/*
typedef	enum
{
	BME280_PATTERN_WeatherMonitoring,
	BME280_PATTERN_HumiditySensing,
	BME280_PATTERN_IndoorNavigation,
	BME280_PATTERN_Gaming
}
EBme280Pattern;
*/

//「整数部・小数部」型
typedef	struct
{
	int16_t	integer;	//負数もあり得る（気温）
	uint8_t	decimal;	//0～9 ← (0.)0～(0.)9 を意味する
}
TBme280IntDec;

//測定値
typedef	struct
{
	TBme280IntDec	temperature;
	TBme280IntDec	humidity;
	TBme280IntDec	pressure;
}
TBme280Value;

class Bme280
{
private:
	uint8_t	slaveAddress = BME280_SLAVE_ADDRESS;

	void	WriteRegValue(uint8_t regAddr, uint8_t dataByte);
	void	ReadRegValue(uint8_t startAddr, uint8_t readLength, uint8_t* retBuf);
	void	DelayMSec(uint8_t msec);
	uint8_t	ComCheck(void);
	void	SoftReset(void);
	void	ReadCalData(void);
	void	WriteSensorParamToReg(void);
	uint8_t	ComputeWaitTime(void);
	uint8_t	GetModulo(int32_t num);
	int32_t		CompensateTemperature(int32_t ut);
	uint32_t	CompensatePressure(uint32_t up);
	uint32_t	CompensateHumidity(int32_t uh);
	void	PresetPatternWeatherMonitoring(void);
	//
	void	BME280_Init(void);
	void	BME280_GetSensorValue(TBme280Value* retValue);

public:
	Bme280(void) {}
	void	initialize(void) { BME280_Init(); }
	void	getSensorValue(TBme280Value* retValue) { BME280_GetSensorValue(retValue); }
};
