
#include <Arduino.h>
#include <WiFi.h>
#include <nvs_flash.h>
#include "NvsWiFiSetting.hpp"

//NVSに保存されているWiFi（STAモード/APモード）のssid/pswdを読み込む
WiFiParam&	NvsWiFiSetting::readWifiParam(wifi_interface_t wi)
{
	const	char* nvsNamespaceWifi = "nvs.net80211";
	Preferences pref;
	pref.begin(nvsNamespaceWifi, true);

	if (wi == WIFI_IF_STA)
	{
		wifiParam.ssid = getParam(&pref, "sta.ssid", 4);	//ssidは4byte目以降がデータ本体
		wifiParam.pswd = getParam(&pref, "sta.pswd");
	}
	else if (wi == WIFI_IF_AP)
	{
		wifiParam.ssid = getParam(&pref, "ap.ssid", 4);	//ssidは4byte目以降がデータ本体
		wifiParam.pswd = getParam(&pref, "ap.pswd");
	}
	else
	{
		wifiParam.ssid = String();
		wifiParam.pswd = String();
	}
	
	pref.end();
	return wifiParam;
}
//
String	NvsWiFiSetting::getParam(Preferences* pref, const char* key, uint8_t valueOffset)
{
	char buf[pref->getBytesLength(key)];
	size_t length = pref->getBytes(key, buf, sizeof(buf));
	return (length <= valueOffset) ? String() : String(&buf[valueOffset]);
}

//デバッグ出力：readWifiParam()で取得した内容を表示する
void	NvsWiFiSetting::debugPrint(const char* label)
{
	WiFiParam sta = readWifiParam(WIFI_IF_STA);	//戻り値を（参照ではなく）構造体のコピーとして受け取る
	WiFiParam ap = readWifiParam(WIFI_IF_AP);	//同上

Serial.println(label);
Serial.printf("sta.ssid:[%s] sta.pswd:[%s]\n", sta.ssid.c_str(), sta.pswd.c_str());
Serial.printf("ap.ssid:[%s] ap.pswd:[%s]\n", ap.ssid.c_str(), ap.pswd.c_str());
}

//NVS全域を消去する
void	NvsWiFiSetting::eraseNvs(void)
{
	debugPrint("erase - before");
	nvs_flash_erase();
	nvs_flash_init();
	debugPrint("erase - after");
}

//WiFi（STAモード）のssid/pswdをNVSへ書き込む
void	NvsWiFiSetting::writeWifiParam(const char* staSsid, const char* staPswd, bool wifiOff)
{
	WiFi.begin(staSsid, staPswd);
	WiFi.disconnect(wifiOff);
}
