//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/08	初版

/*	このクラスの説明
(1)	NVS領域全体を消去する。
	→ESP32により自動保存されているsta.ssid/sta.pswdを消去する目的。
(2)	NVS領域にAP機器のssid/pswd(sta.ssid/sta.pswd)を保存する。
	→ESP32でWebサーバーを立て、ブラウザからAP機器のssid/pswdを受け取り、設定する想定。
	　ssid/pswdをソースコードに埋め込まずにWiFi機能を設定する目的。

NVSのsta.ssid/sta.pswdを更新する手順：
・NVS領域が消去されている状態とする。
・ESP32のWiFiのSTAモードでWiFi.begin(ユーザー指定のssid,同pswd)を実行する。
	→接続成否によらず同ssid,同pswdがNVSのsta.ssid/sta.pswdに保存される。
更新されたことの確認：
・ESP32のWiFiをSTAモードで起動し、WiFi.begin(引数なし)を実行する。
・NVSに保存されたsta.ssid/sta.pswdが読み出され、目的のAP機器に接続される。
	→接続に失敗する場合はAP機器のWSPモード操作（接続ボタン長押し）を試す。
*/

#pragma	once

#include <Arduino.h>
#include <Preferences.h>
#include <WiFi.h>	//wifi_interface_t

struct	WiFiParam
{
	String	ssid, pswd;
};

class	NvsWiFiSetting
{
private:
	WiFiParam	wifiParam;	//readWifiParam()の戻り値はこれを参照する
	String	getParam(Preferences* pref, const char* key, uint8_t valueOffset = 0);

public:
	NvsWiFiSetting(void) {}
	WiFiParam&	readWifiParam(wifi_interface_t wi = WIFI_IF_STA);
	void	writeWifiParam(const char* staSsid, const char* staPswd, bool wifiOff = false);
	void	debugPrint(const char* label);
	void	eraseNvs(void);
};
