//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/01	i2cvfd.h
//	2023/07	i2cvfd.hをFIP9B8.hppへ改造

#include <Arduino.h>
#include <Wire.h>
#include "VfdFIP9B8.hpp"

//初期化
void	VfdFIP9B8::initialize(void)
{
	vfdDataBufCount = 0;
	clear();
}

//セグメントの字形を設定する
void	VfdFIP9B8::setSegmentPattern(void)
{
	//セグメント = 0b-GFEDCBA;
#pragma region	数字0-9
	vfdSeg['0'] = 0b00111111;
	vfdSeg['1'] = 0b00000110;
	vfdSeg['2'] = 0b01011011;
	vfdSeg['3'] = 0b01001111;
	vfdSeg['4'] = 0b01100110;
	vfdSeg['5'] = 0b01101101;
	vfdSeg['6'] = 0b01111101;
	vfdSeg['7'] = 0b00100111;
	vfdSeg['8'] = 0b01111111;
	vfdSeg['9'] = 0b01101111;
#pragma endregion
#pragma region	通信確認
	vfdSeg['W'] = 0b01111110;	//W	字形としては'A'の逆さ
	vfdSeg['i'] = 0b00010000;	//i
	vfdSeg['F'] = 0b01110001;	//F
	vfdSeg['O'] = 0b00111111;	//O
	vfdSeg['K'] = 0b01110101;	//K
	vfdSeg['N'] = 0b01010100;	//n	字形は小文字
	vfdSeg['G'] = 0b00111101;	//G
	vfdSeg['n'] = 0b01010100;	//n
	vfdSeg['t'] = 0b01111000;	//t
	vfdSeg['p'] = 0b01110011;	//P	字形は大文字
#pragma endregion
#pragma region	環境センサーの単位
	vfdSeg['T'] = 0b00110001;	//T	字形としては'「'
	vfdSeg['E'] = 0b01111001;	//E
	vfdSeg['c'] = 0b01011000;	//c
	vfdSeg['H'] = 0b01110110;	//H
//	vfdSeg['U'] = 0b00111110;	//U
	vfdSeg['u'] = 0b00011100;	//u
	vfdSeg['h'] = 0b01110100;	//h
	vfdSeg['P'] = 0b01110011;	//P
	vfdSeg['a'] = 0b01011111;	//a
	vfdSeg['L'] = 0b00111000;	//L
	vfdSeg['x'] = 0b00001000;	//x	字形としては'_'
#pragma endregion
#pragma region	曜日
	vfdSeg['M'] = 0b00110111;	//M	字形としては'U'の逆さ
	vfdSeg['o'] = 0b01011100;	//o
	vfdSeg['e'] = 0b01111011;	//e
	vfdSeg['d'] = 0b01011110;	//d
	vfdSeg['r'] = 0b01010000;	//r
	vfdSeg['S'] = 0b01101101;	//S	字形としては'5'と同じ
#pragma endregion
#pragma region	その他の文字
	vfdSeg['s'] = 0b01101101;	//S	字形は大文字
	vfdSeg['w'] = 0b01111110;	//W	字形は大文字
#pragma endregion
#pragma region	記号
	//セグメント = 0b-GFEDCBA;
	vfdSeg['-'] = 0b01000000;	//-
	vfdSeg[':'] = 0b01001000;	//:	字形としては「=」
	vfdSeg['/'] = 0b01010010;	//／	※「%」の表現に利用
	vfdSeg['_'] = 0b00001000;	//_
	vfdSeg[' '] = 0b00000000;	//空白文字
#pragma endregion
}

//バッファにVFDデータを書き込む（SegmentPatternとMarkPattern）
//・指定のgrid（1桁）に書き込む。
void	VfdFIP9B8::writeBuffer(uint8_t grid, uint8_t segmentPattern, EVfdMark markPattern)
{
	//		:Grid			:MarkPattern	:SegmentPattern					:
	//bit	:7	6	5	4	:3	2	1	0	:7	6	5	4	3	2	1	0	:
	//buf	:[0]							:[1]							:
	
	if (grid == 0 || GridMax < grid) { return; }	//gridがVFDの桁数を超えている場合
	if ((GridMax - 1) * DataBufUnitByte < vfdDataBufCount) { return; }	//バッファに空きがない場合
	vfdDataBuf[vfdDataBufCount++] = ((grid << 4) & 0xF0) | markPattern;
	vfdDataBuf[vfdDataBufCount++] = segmentPattern;
}

//バッファにVFDデータを書き込む（文字列）
//・指定のgridから書き始める。戻り値：最後に書いた文字の右隣のgrid。
//・各文字に対応するSegmentPatternを書き込む。
uint8_t	VfdFIP9B8::writeBuffer(uint8_t grid, String str)
{
	for (uint8_t i = 0; i < str.length(); i++)
	{
		uint8_t segmentPattern = getSegmentPattern(str[i]);
		writeBuffer(grid++, segmentPattern);
	}
	return grid;
}

//バッファにVFDデータを書き込む（MarkPattern）
//・指定のgrid（1桁）に書き込む。
//・SegmentPattern部分は影響を受けない。
void	VfdFIP9B8::writeBuffer(uint8_t grid, EVfdMark markPattern)
{
	if (grid == 0 || GridMax < grid) { return; }	//gridがVFDの桁数を超えている場合
	
	int8_t idx = -1;
	for (uint8_t i = 0; i < vfdDataBufCount; i += DataBufUnitByte)
	{
		if (((vfdDataBuf[i] >> 4) & 0x0F) == grid) { idx = i; }
	}
	if (idx != -1) { vfdDataBuf[idx] = ((grid << 4) & 0xF0) | markPattern; }
}

//表示を更新する（バッファの内容をVFDに送る）
void	VfdFIP9B8::refresh(void)
{
	Wire.beginTransmission(slaveAddress);
	for (uint8_t i = 0; i < vfdDataBufCount; i++) { Wire.write(vfdDataBuf[i]); }
	Wire.endTransmission(true);

	vfdDataBufCount = 0;
}

//表示をクリアする
void	VfdFIP9B8::clear(void)
{
	/*
	Wire.beginTransmission(slaveAddress);
	for (uint8_t grid = 1; grid <= GridMax; grid++)
	{
		Wire.write(((grid << 4) & 0xF0) | VFDMARK_null);
		Wire.write(VFDSEG_null);
	}
	Wire.endTransmission(true);
	*/

	vfdDataBufCount = 0;
	writeBuffer(1, "         ");	//空白がGridMax個
	refresh();
}

//文字に対応するセグメントパターンを取得する
uint8_t	VfdFIP9B8::getSegmentPattern(char cc)
{
/*
	//数値をVFDの数字パターンに変換するテーブル
	static	const	uint8_t	HexSegmentPatternList[] =
		{
			VFDSEG_0, VFDSEG_1, VFDSEG_2, VFDSEG_3, VFDSEG_4, VFDSEG_5, VFDSEG_6, VFDSEG_7, VFDSEG_8, VFDSEG_9,
			VFDSEG_A, VFDSEG_B, VFDSEG_C, VFDSEG_D, VFDSEG_E, VFDSEG_F
		};
	//文字をVFDの文字パターンに変換するテーブル
	static	const	uint8_t	CharSegmentPatternList[] =
		{
			VFDSEG_minus, VFDSEG_colon, VFDSEG_slash, VFDSEG_null,
			VFDSEG_W, VFDSEG_i, VFDSEG_O, VFDSEG_K, VFDSEG_n, VFDSEG_t, VFDSEG_p
		};
	static	String	CharList = String("-: WiOKntp");
	
	uint8_t	segmentPattern = VFDSEG_null;
	if ('0' <= cc && cc <= '9')
	{
		segmentPattern = HexSegmentPatternList[cc - '0'];
	}
	else if ('A' <= cc && cc <= 'F')
	{
		segmentPattern = HexSegmentPatternList[10 + (cc - 'A')];
	}
	else
	{
		for (uint8_t i = 0; i < CharList.length(); i++)
		{
			if (cc != CharList[i]) { continue; }
			segmentPattern = CharSegmentPatternList[i];
			break;
		}
	}

	return segmentPattern;
*/

	//map()でkeyが存在しない場合はvalue(unsigned char)のデフォルト値が取得される。
	//vfdSeg[]では0b00000000が取得され、空白文字を表すセグメントパターンなので、そのまま利用する。
	uint8_t segmentPattern = vfdSeg[cc];
	return segmentPattern;
}
