//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/01	i2cvfd.h
//	2023/07	i2cvfd.hをVfdFIP9B8.hppへ改造

#pragma	once

#include <Arduino.h>
#include <unordered_map>

//スレーブアドレス(7bit)
#define FIP9B8_SLAVE_ADDRESS	0x2C

//VFDの桁数
#define FIP9B8_GRID_MAX	9

//VFDの記号部分
enum	EVfdMark	: uint8_t
{
	VFDMARK_dp			= 0b00000001,	//「.」小数点
	VFDMARK_sweep		= 0b00000010,	//「,」カンマのヒゲ部分
	VFDMARK_apostrophe	= 0b00000100,	//「'」アポストロフィー
	VFDMARK_half		= 0b00001000,	//最小桁にある「1と2」（分数の横線は含まず）
	VFDMARK_comma		= (VFDMARK_dp | VFDMARK_sweep),		//「,」カンマ
	VFDMARK_null		= 0b00000000,	//記号なし
};

//VFD [FIP9B8]
class	VfdFIP9B8
{
private:
	static	const	uint8_t	GridMax = FIP9B8_GRID_MAX;	//VFDの桁数
	uint8_t	slaveAddress = FIP9B8_SLAVE_ADDRESS;

	//セグメントのパターン（字形の辞書）
	std::unordered_map<char, uint8_t> vfdSeg;
	void	setSegmentPattern(void);

	//VFDに転送するデータを格納するバッファ
	static	const	uint8_t	DataBufUnitByte = 2;	//1桁あたり2byte必要
	uint8_t	vfdDataBuf[GridMax * DataBufUnitByte];
	uint8_t	vfdDataBufCount;

public:
	VfdFIP9B8(void) { setSegmentPattern(); }
	void	initialize(void);
	void	writeBuffer(uint8_t grid, uint8_t segmentPattern, EVfdMark markPattern = VFDMARK_null);
	uint8_t	writeBuffer(uint8_t grid, String str);
	void	writeBuffer(uint8_t grid, EVfdMark markPattern);
	void	refresh(void);
	void	clear(void);
	uint8_t	getSegmentPattern(char cc);
	uint8_t	gridMax(void) { return GridMax; }
};
