//	BME280/VFD使用 時計兼環境センサー表示器
//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/08	main()から分離

#include <Arduino.h>
#include "Display.hpp"

//日付を表示する
void	Display::displayDate(const struct tm* tmInfo)
{
	//	|123456789| |123456789|
	//	| 1_1 Mon |-|12_31 Sun|	※月と日の区切りが「-」だと日付であることが分かりにくい
	//	|       . | |        .|
	String strDate = tmToString(tmInfo, "%m_%d %a");
	if (strDate[0] == '0') { strDate[0] = ' '; }	//「月」の十の位が0のとき空白に置き換える
	strDate.replace("_0", "_");		//「日」の十の位が0のとき取り除く
	uint8_t grid = 1;
	grid = vfd.writeBuffer(grid, strDate);	vfd.writeBuffer(grid - 1, VFDMARK_dp);
	grid = vfd.writeBuffer(grid, " ");	//引数のgridが9を超える値で関数を呼び出しても問題ない
	vfd.refresh();

/*	表示案
	//日付	[  1.1. Mon. ] - [ 12.31. Sun.]	※月と日の区切りが「-」「_」だと日付であることが分かりにくい
	String strDate = tmToString(tmInfo, "%m_%d %a");
	uint8_t dpGrid[] = { 3, 5 };	//月と日の「.」の位置
	if (strDate[0] == '0') { strDate[0] = ' '; }	//「月」の十の位が0のとき空白に置き換える
	//strDate.replace("_0", "_");		//「日」の十の位が0のとき取り除く
	if (0 < strDate.indexOf("_0")) { strDate.replace("_0", "_");	dpGrid[1]--; }	//「日」の十の位が0のとき取り除く
	strDate.replace("_", "");
	uint8_t grid = vfd.writeBuffer(2, strDate);	vfd.writeBuffer(grid - 1, VFDMARK_dp);
	vfd.writeBuffer(dpGrid[0], VFDMARK_dp);	vfd.writeBuffer(dpGrid[1], VFDMARK_dp);
	vfd.refresh();
*/
/*	表示案
	//日付	[ 1. 1 Mon] - [12. 31 Sun]	※月と日の区切りがハイフン「-」だと日付であることが分かりにくい
	String strDate = tmToString(tmInfo, "%m_%d %a");
	if (strDate[0] == '0') { strDate[0] = ' '; }	//「月」の十の位が0のとき空白に置き換える
	strDate.replace("_0", "_");		//「日」の十の位が0のとき取り除く
	strDate.replace("_", " ");		//「月_日」の「_」を空白に置き換える
	uint8_t grid = vfd.writeBuffer(1, strDate);	vfd.writeBuffer(grid - 1, VFDMARK_dp);
	vfd.writeBuffer(2, VFDMARK_dp);	//「月.日」の「.」
	vfd.refresh();
*/
}

//時刻を表示する
//・全桁上書きされるようにする（呼び出し元でvfd.clear()不要とするため）。
void	Display::displayTime(const struct tm* tmInfo)
{
	//	|123456789| |123456789|
	//	| 0:00:00 |-|23:59:59 |			↓ "%T" は "%H:%M:%S" と同じ
	String strTime = tmToString(tmInfo, "%T");
	if (strTime[0] == '0') { strTime[0] = ' '; }	//「時」の十の位が0のとき空白に置き換える
	uint8_t grid = 1;
	grid = vfd.writeBuffer(grid, strTime);
	grid = vfd.writeBuffer(grid, " ");
	vfd.refresh();
}

//日時構造体からVFD表示用の文字列に変換する
String	Display::tmToString(const struct tm* tmInfo, const char* fmt)
{
	const	uint8_t bufSize = vfd.gridMax() * 2;	//文字列の'\0'を考慮、かつ、余裕を持ったバッファサイズとする
	char buf[bufSize];
	size_t length = strftime(buf, bufSize, fmt, tmInfo);	//戻り値は'\0'を含まない文字列長
	String strTm = String(buf, length);
//Serial.println(strTm);
	return strTm;
}

//VFDに文字列を表示する（EVfdMarkなしの文字列）
uint8_t	Display::displayString(uint8_t grid, String str)
{
	grid = vfd.writeBuffer(grid, str);
	vfd.refresh();
	return grid;
}

//気温を表示する
//・全桁上書きされるようにする（呼び出し元でvfd.clear()不要とするため）。
void	Display::displayTemperature(const TBme280IntDec* te)
{
	//	|123456789|	|123456789|
	//	| t 234 c |	| t -123 c|
	//	| .  .  ' |	| .   .  '|

	String integer = String(te->integer);
	String decimal = String(te->decimal / 10);
	uint8_t grid = 1;
	grid = vfd.writeBuffer(grid, " t ");	vfd.writeBuffer(grid - 2, VFDMARK_dp);
	grid = vfd.writeBuffer(grid, integer);	vfd.writeBuffer(grid - 1, VFDMARK_dp);
	grid = vfd.writeBuffer(grid, decimal);
	grid = vfd.writeBuffer(grid, " c");	vfd.writeBuffer(grid - 1, VFDMARK_apostrophe);
	if (grid == 9) { vfd.writeBuffer(grid, " "); }
	vfd.refresh();
//Serial.printf("Te. %s.%s 'c\n", integer, decimal);
}

//湿度を表示する
//・全桁上書きされるようにする（呼び出し元でvfd.clear()不要とするため）。
void	Display::displayHumidity(const TBme280IntDec* hu)
{
	//	|123456789|
	//	|Hu 567 / |
	//	| .  .  % |	「%」の丸印部分を1桁内の「'」「.」で表現する
	
	String integer = String(hu->integer);
	String decimal = String(hu->decimal / 10);
	uint8_t grid = 1;
	grid = vfd.writeBuffer(grid, "Hu ");	vfd.writeBuffer(grid - 2, VFDMARK_dp);
	grid = vfd.writeBuffer(grid, integer);	vfd.writeBuffer(grid - 1, VFDMARK_dp);
	grid = vfd.writeBuffer(grid, decimal);
	grid = vfd.writeBuffer(grid, " /");	vfd.writeBuffer(grid - 1, (EVfdMark)(VFDMARK_apostrophe | VFDMARK_dp));
	grid = vfd.writeBuffer(grid, " ");
	vfd.refresh();
//Serial.printf("Hu. %s.%s %%\n", integer, decimal);
}

//気圧を表示する
//・全桁上書きされるようにする（呼び出し元でvfd.clear()不要とするため）。
void	Display::displayPressure(const TBme280IntDec* pr)
{
	//	|123456789|
	//	| 1024 hPa|
	//	|  987 hPa|

	String blank = (1000 <= pr->integer) ? " " : "  ";
	String integer = String(pr->integer);
	displayString(1, blank + integer + " hPa");
//Serial.printf("Pu. %s hPa\n", integer);
}

//照度を表示する
void	Display::displayLux(uint16_t lux)
{
	//	|123456789|
	//	|65535 Lux|
	//	|    0 Lux|

	char	buf[20];
	sprintf(buf, "%5d Lux\0", lux);
	displayString(1, buf);
//Serial.printf("Lux %s \n", buf);
}

//IP-Addressを表示する
//・IPAddressをポインタ引数にすると (*addr)[0] などと書かなくてはならず面倒くさいので参照引数とする。
void	Display::displayIPAddress(const IPAddress& addr)
{
	for (uint8_t grid = 1, i = 0; i < 4; i++)
	{
		grid = vfd.writeBuffer(grid, String(addr[i]));
		if (i < 3) { vfd.writeBuffer(grid - 1, VFDMARK_dp); }
	}
	vfd.refresh();
}

//現在の時刻(秒)に応じて時刻やセンサー値を表示する
void	Display::displayTimeAndSensorValue(const struct tm& timeInfo, const TBme280Value& tValue, const uint16_t lux)
{
	//日時やセンサー値の表示を更新する
	//時刻
	if (0 <= timeInfo.tm_sec && timeInfo.tm_sec <= 5	||
		26 <= timeInfo.tm_sec && timeInfo.tm_sec <= 35	||
		56 <= timeInfo.tm_sec && timeInfo.tm_sec <= 59)
	{
		displayTime(&timeInfo);
	}

	//日付
	if (6 <= timeInfo.tm_sec && timeInfo.tm_sec < 6 + 5	||
		36 <= timeInfo.tm_sec && timeInfo.tm_sec < 36 + 5)
	{
		displayDate(&timeInfo);
	}

#if (0)	//気温・湿度・気圧
	//気温
	if (11 <= timeInfo.tm_sec && timeInfo.tm_sec < 11 + 5	||
		41 <= timeInfo.tm_sec && timeInfo.tm_sec < 41 + 5)
	{
		displayTemperature(&tValue.temperature);
	}

	//湿度
	if (16 <= timeInfo.tm_sec && timeInfo.tm_sec < 16 + 5	||
		46 <= timeInfo.tm_sec && timeInfo.tm_sec < 46 + 5)
	{
		displayHumidity(&tValue.humidity);
	}

	//気圧
	if (21 <= timeInfo.tm_sec && timeInfo.tm_sec < 21 + 5	||
		51 <= timeInfo.tm_sec && timeInfo.tm_sec < 51 + 5)
	{
		displayPressure(&tValue.pressure);
	}
#else	//気温・湿度・気圧・照度
	//気温
	if (11 <= timeInfo.tm_sec && timeInfo.tm_sec < 11 + 4	||
		41 <= timeInfo.tm_sec && timeInfo.tm_sec < 41 + 4)
	{
		displayTemperature(&tValue.temperature);
	}

	//湿度
	if (15 <= timeInfo.tm_sec && timeInfo.tm_sec < 15 + 4	||
		45 <= timeInfo.tm_sec && timeInfo.tm_sec < 45 + 4)
	{
		displayHumidity(&tValue.humidity);
	}

	//気圧
	if (19 <= timeInfo.tm_sec && timeInfo.tm_sec < 19 + 4	||
		49 <= timeInfo.tm_sec && timeInfo.tm_sec < 49 + 4)
	{
		displayPressure(&tValue.pressure);
	}

	//照度
	if (23 <= timeInfo.tm_sec && timeInfo.tm_sec < 23 + 3	||
		53 <= timeInfo.tm_sec && timeInfo.tm_sec < 53 + 3)
	{
		displayLux(lux);
	}
#endif
}
