//	BME280/VFD使用 時計兼環境センサー表示器
//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/09	新規作成

#pragma once

#include <Arduino.h>
#include <SPIFFS.h>
#include <WebServer.h>
#include "Bme280.hpp"

/*	You only need to format SPIFFS the first time you run a
	test or else use the SPIFFS plugin to create a partition
	https://github.com/me-no-dev/arduino-esp32fs-plugin	*/
#define	FORMAT_SPIFFS_IF_FAILED	true

//ファイル名
#define	RECORD_FILENAME_CSV	"/sensor.csv"
#define	RECORD_FILENAME_BAK	"/sensor.bak"
#define	RECORD_UPDATE_TIME	"/updatetime.txt"
#define	GRAPH_FILENAME_HTML	"/graph.html"	//未使用

//保存するレコード件数（1時間ごとに1件）
#define	RECORD_COUNT_MAX	48

//グラフ描画に使用するデータ（センサー値）
class	GraphData
{
public:
	String	jointData;	//「,」で繋いだデータ（センサー値または日時）
	float	max, min;	//センサー値の最大値、最小値
	int16_t	scaleMax, scaleMin;	//グラフの目盛りの最大値、最小値（適当に切りのよい値）
	String	lastData;	//グラフ外に表示するための最新データ（センサー値または日時）

	void	initialize(void)
	{
		jointData = "";
		max = std::numeric_limits<float>::min();	//比較元の初期値なのでmaxに対してはmin()
		min = std::numeric_limits<float>::max();	//比較元の初期値なのでminに対してはmax()
		scaleMax = scaleMin = 0;
		lastData = "";
	}
};

//グラフ描画に使用するデータ（Y軸の値）と日時（X軸の値）
class	GraphDataGroup
{
public:
	GraphData	temperature;
	GraphData	humidity;
	GraphData	pressure;
	GraphData	dateTime;
};

//センサー値の記録を読み書きするクラス
class	Record
{
private:
	const	char*	fileCsv = RECORD_FILENAME_CSV;
	const	char*	fileBak = RECORD_FILENAME_BAK;
	const	char*	fileUpdateTime = RECORD_UPDATE_TIME;
	const	char*	fileGraphHtml = GRAPH_FILENAME_HTML;	//未使用
	const	uint8_t	recordCountMax = RECORD_COUNT_MAX;
	String	lastUpdateTimestamp = "";

	String	createRecord(const struct tm& tTimeInfo, const TBme280Value& tSensorValue);
	std::vector<String> readCsv(void);
	void	fileAvailable(void);
	bool	update(const String& record);
	String	buildTimestamp(const struct tm& tTimeInfo);
	String	readTimestamp(void);
	bool	writeTimestamp(const String& timestamp);
	bool	writeFile(const String& filename, const String& data);

	//CSV(レコード)のフォーマット
	//				"yyyy/mm/dd","hh:mm","tt.t","hh.h","pppp.p"	//年月日,時分,温度,湿度,気圧
	//fieldIndex	[0]			[1]		[2]		[3]		[4]
	const	uint8_t	fieldIdxYMD = 0, fieldIdxHM = 1, fieldIdxT = 2, fieldIdxH = 3, fieldIdxP = 4;
	void	buildGraphDataset(const std::vector<std::vector<String>>& fields, GraphData& graphData, const uint8_t fieldIdx);
	void	buildGraphLabel(const std::vector<std::vector<String>>& fields, GraphData& graphData);

public:
	GraphDataGroup	graphData;

	Record(void) { SPIFFS.begin(FORMAT_SPIFFS_IF_FAILED);	lastUpdateTimestamp = readTimestamp(); }
	void	write(const struct tm& timeInfo, const TBme280Value& sensorValue);
	const	char*	csvFilename(void) { return fileCsv; }
	const	char*	graphHtmlFilename(void) { return fileGraphHtml; }
	void	deleteFile(void);
	void	loadGraphData(void);
	bool	isUpdatedThisHour(const struct tm& timeInfo);
	bool	saveGraphHtmlFile(String& graphHtml);	//未使用
	String	graphDownloadName(void);	//未使用
};

//Stringのsplit()実装のため
#include <iostream>
#include <string>
#include <vector>
#include <sstream>
//
class	StrUty
{
public:
	static	std::vector<String>	split(String& src, const char separator)
	{
		std::vector<String> dist;
		std::stringstream ss(src.c_str());
		std::string buf;
		while (std::getline(ss, buf, separator))
		{
			dist.push_back(String(buf.c_str()));
		}
		return dist;
	}
};
