//	『昼夜逆転』工作室	http://jsdiy.starfree.jp
//	2023/08	初版

/*	このクラスの説明
ESP32でWebサーバーを立て、ブラウザからAP機器のssid/pswdを設定する。
	→ssid/pswdをソースコードに埋め込まずにWiFi機能を設定する目的。

手順：
ESP32のWiFiをAP/STA両立モードで起動し、APモードの立場でWebサーバーを立てる。
PC/スマホのWiFi接続画面でESP32（APモード）のssid/pswdを指定し、接続を確立する。
	→既存の接続からの切り替えに数十秒かかる場合あり。
PC/スマホのブラウザでWebサーバーへアクセスする(http://192.186.4.1)。
表示されたWebページで、ESP32のWiFiのSTAモードで接続したいAP機器のssid/pswdを入力し、POSTする。
Webサーバーで受信し、NVSのsta.ssid/sta.pswdを更新する。
Webブラウザからの操作は完了。ESP32を再起動する（電源再投入）。

＜ESP32再起動後＞（アプリ側プログラムでの手順）
ESP32のWiFiをSTAモードで起動し、WiFi.begin(引数なし)を実行する。
NVSに保存されたsta.ssid/sta.pswdが読み出され、目的のAP機器に接続される。
	→接続に失敗する場合はAP機器のWSPモード操作（接続ボタン長押し）を試す。
*/

#pragma	once

#include <Arduino.h>
#include <WiFi.h>	//IPAddress

//ESP32をAPとして公開する際のSSIDとパスワード(KEY)
#define	ESP32_WIFI_AP_SSID	"ESP32-AP-5a1e7026"		//後ろの数字は乱数
#define	ESP32_WIFI_AP_PSWD	"pswd1234"

class	WiFiApServer
{
private:
	const	char*	apSsid	= ESP32_WIFI_AP_SSID;
	const	char*	apPswd	= ESP32_WIFI_AP_PSWD;

public:
	WiFiApServer(void) {}
	IPAddress	serverSetup(void);
	void	listening(void);
	const	char*	pswd(void) { return apPswd; }
};
