
#pragma	once

#include <Arduino.h>

#pragma region	//htmlDocTemplate
//・'u8' はUTF-8エンコードの文字列とすることの指定。
//・文字列中に '(' と ')' を使用できるよう、リテラル文字列の範囲を '**(' から ')**' までと定義した。
const	String	htmlDocTemplate = u8R"**(
<!DOCTYPE html>
<html lang="ja">
<head>
<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0" />
<title>{ESP32-Product}</title>
</head>
<body>
{BODY}
</body>
</html>
)**";
#pragma endregion

#pragma region	//root/
const	String	rootBody = u8R"**(
<h1>センサー値</h1>
<p>
■<a href="/sv/?sval=graph">センサー値を表示する</a><br>　画面にグラフを表示します。
</p>
<p>
■CSVファイルを【<a href="{FILE_CSV}" download>ダウンロード</a>】する<br>　データをPCで利用したい場合はこちら。
</p>
<p>
■<a href="/sv/?sval=delete">センサー値を削除する</a><br>　過去データを削除します。
</p>
<h1>アクセスポイント登録</h1>
<p>
■<a href="/i/?scan=yes">近くのネットワークを探す</a><br>　リンク押下後5秒程度お待ちください。
</p>
<p>
■<a href="/i/?scan=no">ネットワーク名を自分で入力する</a><br>　SSIDを非表示にしている場合はこちら。
</p>
<p>
■<a href="/e/?erase=yes">ネットワーク情報を消去する</a><br>　登録済みの情報を消去する場合はこちら。
</p>
)**";
#pragma endregion

#pragma region	//root/i/
String	inputParamBody = u8R"**(
<h1>アクセスポイント登録</h1>
<p>■{ESP32-Product}がWiFi接続するアクセスポイントを指定してください。</p>
<form action="/i/" method="post" style="line-height: 1.5em">
	<label for="id_ssid">SSID:</label><br>
	{SSID_PART}<br>
	<label for="id_key">KEY(password):</label><br>
	<input type="password" name="userkey" id="id_key" required><br>
	<br>
	<input type="submit" value="{ESP32-Product}へ登録する">
</form>
<p style="line-height: 2em">
<a href="/">トップページへ戻る</a>
</p>
)**";
/*
	<input type="submit" value="{ESP32-Product}へ登録する"
		style="width: 9em; white-space: pre-line; word-break: break-all;">
		↑submitボタン内でテキストを折り返す方法
*/
//{SSID_PART}には<select>か<input>が入る
#pragma endregion

#pragma region	//root/i/の<select>
String	inputParamSelect = u8R"**(
<select name="userssid" id="id_ssid" required>
<option value="">選択してください</option>
{OPTION}
</select><br>
)**";
//{OPTION}には<option value=xxx>が入る（コードで生成する）
#pragma endregion

#pragma region	//root/i/の<input>
const	String	inputParamInput = u8R"**(
<input type="text" name="userssid" id="id_ssid" required><br>
)**";
#pragma endregion

#pragma region	//root/i(POST)/
const	String	updateParamBody = u8R"**(
<h1>登録完了</h1>
<p>
■{ESP32-Product}を再起動し、WiFi接続が成功することを確認してください。
WiFi接続に失敗する場合はアクセスポイントをWPSモードにし、{ESP32-Product}を再起動してください。
</p>
<p>
■それでもWiFi接続に失敗する場合、アクセスポイントの登録をやり直します。
その際はブラウザを閉じ、スマートホンの{ESP32-Product}へのネットワーク接続も一旦解除してください。
</p>
<p>
■{ESP32-Product}を再起動せずにアクセスポイントの登録をやり直す場合、下記リンクからトップページへ戻ってください。
</p>
<p>
[参考] WPSモードの開始方法<br>
ルーター/アクセスポイント機器の「AOSS」「らくらく無線スタート」などの名称のボタンを数秒押します。
</p>
<form action="/r/" method="post">
	<input type="submit" name="restart" value="{ESP32-Product}を再起動する">
</form>
<p style="line-height: 2em">
<a href="/">トップページへ戻る</a>
</p>
)**";
/*
	<input type="submit" name="restart" value="{ESP32-Product}を再起動する"
		style="width: 9em; white-space: pre-line; word-break: break-all;">
		↑submitボタン内でテキストを折り返す方法
*/
#pragma endregion

#pragma region	//root/e(GET)/
const	String	dataEraseNvsBody = u8R"**(
<h1>ネットワーク情報消去完了</h1>
<p>
登録されたネットワーク名／パスワードを消去しました。
トップページへ戻り、アクセスポイントを登録し直してください。
</p>
<p>
<a href="/">トップページへ戻る</a>
</p>
)**";
#pragma endregion

#pragma region	//root/sv/?sval=delete
const	String	dataDeleteBody = u8R"**(
<h1>データ削除完了</h1>
<p>
センサー値のデータを削除しました。
</p>
<p>
<a href="/">トップページへ戻る</a>
</p>
)**";
#pragma endregion

#pragma region	//root/sv/?sval=graph
const	String	chartBody = u8R"**(
<p>
{LAST_DATETIME}<br>
気温:{LAST_TEMPERATURE}℃　湿度:{LAST_HUMIDITY}%　気圧:{LAST_PRESSURE}hPa
</p>
<canvas id = "chartTem"></canvas>
<canvas id = "chartHu"></canvas>
<canvas id = "chartPres"></canvas>
<script src = "/chart.umd.js"></script>
<script>
var ctx = document.getElementById("chartTem");
var chart = new Chart(ctx, {
	type: 'line',	//折れ線グラフ
	data: {
		labels: [{LABEL_DATETIME}],
		datasets: [
			{
				label: "気温(℃)",
				data: [{DATA_TEMPERATURE}],
				borderColor: 'red',
				lineTension: 0.3,	//折れ線の角の曲がり具合（曲線グラフになる）
				pointRadius: 0,		//折れ線の角に打つ丸印の半径（0は丸印なし）
				borderWidth: 2,		//線の太さ(px)
				fill: true,			//グラフ内部を塗る（折れ線の場合はX軸と挟む領域）
				backgroundColor: 'rgba(255, 128, 128, 0.2)',	//グラフ内部色
				yAxisID: 'yTemperature',
			},
		],
	},
	options: {
		bezierCurve : true,
		scales: {
			x: {
				//垂直線を非表示（垂直線はX軸に対するグリッドの設定なので「x:」に記述する）
				grid: { display: false },
			},
			yTemperature: {
				max: {SCALE_TEMPERATURE_MAX},
				min: {SCALE_TEMPERATURE_MIN},
				grace: "10%",	//max,minの範囲外に持たせる余裕（余分に振る目盛り）
				ticks: { color: 'red' },	//軸の文字色
				grid: { color: 'rgba(255, 0, 0, 0.2)' },	//水平線の色
			},
		}
	}
});
</script>
<script>
var ctx = document.getElementById("chartHu");
var chart = new Chart(ctx, {
	type: 'line',
	data: {
		labels: [{LABEL_DATETIME}],
		datasets: [
			{
				label: '湿度(%)',
				data: [{DATA_HUMIDITY}],
				borderColor: 'blue',
				lineTension: 0.3,
				pointRadius: 0,
				borderWidth: 2,
				fill: true,
				backgroundColor: 'rgba(128, 128, 255, 0.2)',
				yAxisID: 'yHumidity',
			},
		],
	},
	options: {
		bezierCurve : true,
		scales: {
			x: {
				grid: { display: false },
			},
			yHumidity: {
				max: {SCALE_HUMIDITY_MAX},
				min: {SCALE_HUMIDITY_MIN},
				grace: "10%",
				ticks: { color: 'blue' },
				grid: { color: 'rgba(0, 0, 255, 0.2)' },
			},
		}
	}
});
</script>
<script>
var ctx = document.getElementById("chartPres");
var chart = new Chart(ctx, {
	type: 'line',
	data: {
		labels: [{LABEL_DATETIME}],
		datasets: [
			{
				label: '気圧(hPa)',
				data: [{DATA_PRESSURE}],
				borderColor: 'Green',
				lineTension: 0.3,
				pointRadius: 0,
				borderWidth: 2,
				fill: true,
				backgroundColor: 'rgba(128, 255, 128, 0.2)',
				yAxisID: 'yPressuer',
			},
		],
	},
	options: {
		bezierCurve : true,
		scales: {
			x: {
				grid: { display: false },
			},
			yPressuer: {
				max: {SCALE_PRESSURE_MAX},
				min: {SCALE_PRESSURE_MIN},
				grace: "10%",
				ticks: { color: 'green' },
				grid: { color: 'rgba(0, 255, 0, 0.2)' },
			},
		}
	}
});
</script>
)**";
#pragma endregion
